# encoding: utf-8
require 'date'

module Ginbee
  module Dbbackup
    # レセプト日次分データを送信する(統合ストレージ格納用)
    # run_mis.shを実行すると先月頭から当月の指定日付までのデータが生成されるので、
    # 毎月1日に生成と送信を行う
    # 送信失敗時の再送は行っていないが、翌日のdaily cronで再チャレンジされる想定
    #
    # stateファイルについて:
    # - :state
    #   - "INITIALIZED"
    #     作業を開始した
    #   - "MIS_DATA_GENERATED"
    #     データ生成まで完了した
    #   - "MIS_ARCHIVE_CREATED"
    #     tarで固めるところまで完了した
    #   - "SUCCESS"
    #     生成と送信に成功した
    #   - "FAILED"
    #     生成はできたが送信に失敗した
    # - :month
    #   - "2018-11-01"のような文字列
    class MisUploader < Uploader
      def initialize(*args)
        super
        raise 'mis_command not set' if Dbbackup.conf['mis_command'].nil?
      end

      def self.state_file_path
        return File.join(Dbbackup.conf['work_dir'], "mis_upload.state")
      end

      def need_upload?
        if state[:state] == "SUCCESS" && state[:month] == this_month.to_s
          Dbbackup.logger.debug("already sent mis data this month")
          return false
        end
        if receipt_data_output_enabled?
          true
        else
          Dbbackup.logger.debug("receipt_data_output not enabled for this tenant")
          false
        end
      end

      def run
        setup
        generate_mis_data
        archive_mis_data
        send_file(archive_path)
        cleanup
      ensure
        dump_state
      end

      private

      # サーバ側に設定を問い合わせる
      def receipt_data_output_enabled?
        query = Dbbackup.conf["dbbackup_api_key"] ? "?dbbackup_api_key=#{Dbbackup.conf['dbbackup_api_key']}" : ""
        url = File.join(Dbbackup.conf['sms_host'], ["/dbbackup_service/api/server", query].join)
        res = @client.get(url, '', 'Content-Type' => 'application/json')
        unless HTTP::Status.successful?(res.status)
          raise "failed to check mis upload is needed: http_response=<#{res.inspect}>"
        end
        json = JSON.parse(res.content)
        return json['output_receipt_data']
      end

      # 一時ディレクトリの用意
      def setup
        state[:state] = "INITIALIZED"
        state[:month] = this_month.to_s
        FileUtils.rm_r(tmp_dir) if File.exist?(tmp_dir)
        FileUtils.mkdir_p(tmp_dir)
        FileUtils.rm(archive_path) if File.exist?(archive_path)
      end

      # データ生成を実行する
      def generate_mis_data
        with_umask do
          run_command Dbbackup.conf['mis_command'].gsub('{tmp_dir}', tmp_dir)
        end
        if Dir.entries(tmp_dir).sort == ['.', '..']
          raise "mis_command did not generate any files"
        end

        state[:state] = "MIS_DATA_GENERATED"
        state[:month] = this_month.to_s
      end

      # tarで固める
      def archive_mis_data
        with_umask do
          run_command "tar", "zcf", archive_path, tmp_dir
        end
        run_command "rm", "-r", tmp_dir
        state[:state] = "MIS_ARCHIVE_CREATED"
        state[:month] = this_month.to_s
      end

      # 送信済みファイルの削除
      def cleanup
        run_command "rm", archive_path
      end

      def state_file
        return MisUploader.state_file_path
      end

      def run_command(*cmd)
        o,e,s = Open3.capture3(*cmd)
        Dbbackup.logger.debug("executed cmd: cmd=<#{cmd}>,out=<#{o}>,err=<#{e}>")
        if not s.success?
          raise "command error: process_status=<#{s.inspect},stdout=<#{o}>,stderr=<#{e}>"
        end
      end

      # umaskを生成ファイルが600になるよう設定してブロックを実行する
      def with_umask(&block)
        orig_umask = File.umask(0077)
        block.call
      ensure
        File.umask(orig_umask)
      end

      # 一時データ生成用ディレクトリ
      def tmp_dir
        File.join(Dbbackup.conf['work_dir'], 'mis')
      end

      # 送信用アーカイブ作成パス
      def archive_path
        File.join(Dbbackup.conf['work_dir'], 'mis.tar.gz')
      end

      # dayの属する月を返す(月はday=1のDateで表す)
      def month_of(day)
        Date.new(day.year, day.month, 1)
      end

      # 現在の月を返す
      # 二回以上呼び出しても必ず同じ値を返す(実行中に月が変わっても変にならないように)
      def this_month
        @this_month ||= month_of(Date.today)
      end
    end
  end
end
