# encoding: utf-8
require 'json'
require 'uri'
require 'digest/sha1'

module Ginbee
  module Dbbackup
    # BASEバックアップを分割して送信する
    class SplitBaseUploader < BaseUploader
      INTERVAL_DAYS = 7

      def initialize(url_path, cert, cert_key, cacert)
        super
        @split_size = Dbbackup.conf['split_size'] || "10m" # 10MB
      end

      def run
        # 強制的にフルバックアップを実行
        if Dbbackup.conf['force_fullbackup'] == true
          return full_backup
        end
        if state[:state] == "FAILED"
          if state[:retry_count].to_i < @retry_limit
            # 再送信をカウント
            state[:retry_count] = state[:retry_count].to_i + 1
            Dbbackup.logger.debug("split base reupload: retry_count=<#{state[:retry_count]}>")
            # 失敗した場合は途中から送信
            return split_send_file(split_backup_dir)
          else
            # リトライ上限を超えた場合は送信を諦める
            clear_split_base_file
            return false
          end
        else
          return full_backup
        end
      ensure
        dump_state
      end

      private
      # フルバックアップ実行
      def full_backup
        # BASEバックアップを作成し分割して送信
        base_backup_file = create_backup_file
        split_base_file(base_backup_file)
        return split_send_file(split_backup_dir)
      end

      # 分割したファイルを送る
      def split_send_file(dir)
        now_file = state[:filename].to_s
        max = Dir.glob("#{split_backup_dir}/split.*").size - 1
        base_sha1 = state[:base_sha1]
        Dir.glob("#{split_backup_dir}/split.*").sort.each_with_index do |file, i|
          # 送信済みファイルは無視する
          if File.basename(file) >= now_file
            send_file(file, {
              "HTTP-X-Backup-CONCAT-SHA1" => base_sha1,
              "HTTP-X-Backup-Split-Index" => i,
              "HTTP-X-Backup-Split-Max" => max,
            })
          end
        end
      end

      # BASEファイルを分割する
      def split_base_file(file)
        clear_split_base_file
        cmd = "split -b #{@split_size} --additional-suffix .gz #{base_backup_file} -a 8 -d #{split_backup_dir}/split."
        o,e,s = Open3.capture3(cmd)
        Dbbackup.logger.debug("executed split: cmd=<#{cmd}>,out=<#{o}>,err=<#{e}>")
        if not s.success?
          raise "split command error: process_status=<#{s.inspect},stdout=<#{o}>,stderr=<#{e}>"
        end
        state[:base_sha1] = Digest::SHA1.file(file).to_s
      end

      def state_file
        return File.join(Dbbackup.conf['work_dir'], "split_base_upload.state")
      end

      def split_backup_dir
        return File.join(Dbbackup.conf['work_dir'], "ginbee_split_base_backup")
      end

      def clear_split_base_file
        # 以前の情報をクリアする
        state.clear
        FileUtils.mkdir_p(split_backup_dir())
        FileUtils.rm_f(Dir.glob("#{split_backup_dir()}/split.*"))
      end

      # テストでスタブ化するため定義
      def time_now
        return Time.now
      end
    end
  end
end
